# Generated by Django 5.2.9 on 2025-12-22 06:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("relations", "0003_relation_relations_r_subj_content_type_and_more"),
        ("sample_project", "0001_squashed_0011_alter_place_feature_code_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="IsParentOf",
            fields=[
                (
                    "relation_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="relations.relation",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("relations.relation",),
        ),
    ]
