{% extends "generic/genericmodel_list.html" %}

{% block title %}
  {{ object_list.model.name_and_reverse_name|title }}
{% endblock title %}

{% block listviewheader %}
  {{ object_list.model.name_and_reverse_name|title }}
{% endblock listviewheader %}
