<!DOCTYPE html>
{% load static %}
<html lang="en">
  <link href="{% static 'css/maintenance.css' %}" rel="stylesheet" />
  <head>
    <meta charset="UTF-8">
    <meta name="description" content="Site Maintenance Page">
    <title>403 Forbidden</title>
  </head>
  <body>
    <article>
      <h1>403 Forbidden</h1>
      <div>
        <p>
          You don't have permissions to access to this resource. Please contact the instance technician to discuss possible solutions.
        </p>
      </div>
    </article>
  </body>
</html>
